package micha

const (
	PARSE_MODE_DEFAULT  ParseMode = ""
	PARSE_MODE_HTML     ParseMode = "HTML"
	PARSE_MODE_MARKDOWN ParseMode = "Markdown"

	CHAT_TYPE_PRIVATE    ChatType = "private"
	CHAT_TYPE_GROUP      ChatType = "group"
	CHAT_TYPE_SUPERGROUP ChatType = "supergroup"
	CHAT_TYPE_CHANNEL    ChatType = "channel"

	CHAT_ACTION_TYPING          ChatAction = "typing"
	CHAT_ACTION_UPLOAD_PHOTO    ChatAction = "upload_photo"
	CHAT_ACTION_RECORD_VIDEO    ChatAction = "record_video"
	CHAT_ACTION_UPLOAD_VIDEO    ChatAction = "upload_video"
	CHAT_ACTION_RECORD_AUDIO    ChatAction = "record_audio"
	CHAT_ACTION_UPLOAD_AUDIO    ChatAction = "upload_audio"
	CHAT_ACTION_UPLOAD_DOCUMENT ChatAction = "upload_document"
	CHAT_ACTION_FIND_LOCATION   ChatAction = "find_location"

	MEMBER_STATUS_CREATOR       MemberStatus = "creator"
	MEMBER_STATUS_ADMINISTRATOR MemberStatus = "administrator"
	MEMBER_STATUS_MEMBER        MemberStatus = "member"
	MEMBER_STATUS_LEFT          MemberStatus = "left"
	MEMBER_STATUS_KICKED        MemberStatus = "kicked"

	MESSAGE_ENTITY_MENTION      MessageEntityType = "mention"
	MESSAGE_ENTITY_HASHTAG      MessageEntityType = "hashtag"
	MESSAGE_ENTITY_BOT_COMMAND  MessageEntityType = "bot_command"
	MESSAGE_ENTITY_URL          MessageEntityType = "url"
	MESSAGE_ENTITY_EMAIL        MessageEntityType = "email"
	MESSAGE_ENTITY_BOLD         MessageEntityType = "bold"
	MESSAGE_ENTITY_ITALIC       MessageEntityType = "italic"
	MESSAGE_ENTITY_CODE         MessageEntityType = "code"
	MESSAGE_ENTITY_PRE          MessageEntityType = "pre"
	MESSAGE_ENTITY_TEXT_LINK    MessageEntityType = "text_link"
	MESSAGE_ENTITY_TEXT_MENTION MessageEntityType = "text_mention"
)

type ParseMode string
type ChatType string
type ChatAction string
type MemberStatus string
type MessageEntityType string

// User object represents a Telegram user, bot
type User struct {
	ID        int64  `json:"id"`
	FirstName string `json:"first_name"`
	LastName  string `json:"last_name"`
	Username  string `json:"username"`
}

type ChatID string

func (chatID *ChatID) UnmarshalJSON(value []byte) error {
	*chatID = ChatID(value)
	return nil
}

// Chat object represents a Telegram user, bot or group chat.
type Chat struct {
	ID        ChatID   `json:"id"`
	Type      ChatType `json:"type"`
	Title     string   `json:"title"`
	FirstName string   `json:"first_name"`
	LastName  string   `json:"last_name"`
	Username  string   `json:"username"`
}

type ChatMember struct {
	User   User         `json:"user"`
	Status MemberStatus `json:"status"`
}

type FileBase struct {
	FileID   string `json:"file_id"`
	FileSize uint64 `json:"file_size"`
}

// Thumbnail object represents an image/sticker of a particular size.
type PhotoSize struct {
	FileBase

	Width  int `json:"width"`
	Height int `json:"height"`
}

// Photo object represents a photo with caption.
type Photo struct {
	FileBase
	PhotoSize

	Caption string `json:"caption"`
}

// Audio object represents an audio file (voice note).
type Audio struct {
	FileBase

	Duration  int    `json:"duration"`
	MimeType  string `json:"mime_type"`
	Performer string `json:"performer"`
	Title     string `json:"title"`
}

// Document object represents a general file (as opposed to Photo or Audio).
// Telegram users can send files of any type of up to 1.5 GB in size.
type Document struct {
	FileBase

	Thumb    *PhotoSize `json:"thumb"`
	FileName string     `json:"file_name"`
	MimeType string     `json:"mime_type"`
}

// Sticker object represents a WebP image, so-called sticker.
type Sticker struct {
	FileBase

	Width  int        `json:"width"`
	Height int        `json:"height"`
	Thumb  *PhotoSize `json:"thumb"`
	Emoji  string     `json:"emoji"`
}

// Video object represents an MP4-encoded video.
type Video struct {
	FileBase

	Duration int        `json:"duration"`
	Width    int        `json:"width"`
	Height   int        `json:"height"`
	Thumb    *PhotoSize `json:"thumb"`
}

// Voice object represents a voice note.
type Voice struct {
	FileID   string `json:"file_id"`
	Duration int    `json:"duration"`
	MimeType string `json:"mime_type"`
	FileSize int    `json:"file_size"`
}

// Contact object represents a contact to Telegram user
type Contact struct {
	UserID      int64  `json:"user_id"`
	PhoneNumber string `json:"phone_number"`
	FirstName   string `json:"first_name"`
	LastName    string `json:"last_name"`
}

// Location object represents geographic position.
type Location struct {
	Longitude float64 `json:"longitude"`
	Latitude  float64 `json:"latitude"`
}

// Venue object represents a venue.
type Venue struct {
	Location     Location `json:"location"`
	Title        string   `json:"title"`
	Address      string   `json:"address"`
	FoursquareID string   `json:"foursquare_id"`
}

type UserProfilePhotos struct {
	TotalCount int           `json:"total_count"`
	Photos     [][]PhotoSize `json:"photos"`
}

type File struct {
	FileBase
	FilePath string `json:"file_path"`
}

type MessageEntity struct {
	Type   MessageEntityType `json:"type"`
	Offset int               `json:"offset"`
	Limit  int               `json:"limit"`

	// Optional
	URL  string `json:"url"`  // For “text_link” only, url that will be opened after user taps on the text
	User *User  `json:"user"` // For “text_mention” only, the mentioned user
}

// You can provide an animation for your game so that it looks stylish in chats.
// This object represents an animation file to be displayed in the message containing a game.
type Animation struct {
	FileID string `json:"file_id"`

	// Optional
	Thumb    *PhotoSize `json:"thumb"`
	FileName string     `json:"file_name"`
	MimeType string     `json:"mime_type"`
	FileSize *int       `json:"file_size"`
}

// This object represents a game.
// Use BotFather to create and edit games, their short names will act as unique identifiers.
type Game struct {
	Title       string      `json:"title"`
	Description string      `json:"description"`
	Photo       []PhotoSize `json:"photo"`

	// Optional
	Text         string          `json:"text"`
	TextEntities []MessageEntity `json:"text_entities"`
	Animation    *Animation      `json:"animation"`
}

// This object represents one row of the high scores table for a game.
type GameHighScore struct {
	Position int  `json:"position"`
	User     User `json:"user"`
	Score    int  `json:"score"`
}

// Message object represents a message.
type Message struct {
	MessageID int64  `json:"message_id"`
	From      User   `json:"from"`
	Date      uint64 `json:"date"`
	Chat      Chat   `json:"chat"`

	// Optional
	ForwardFrom           *User           `json:"forward_from"`
	ForwardFromChat       *Chat           `json:"forward_from_chat"`
	ForwardFromMessageID  int64           `json:"forward_from_message_id"`
	ForwardDate           uint64          `json:"forward_date"`
	ReplyToMessage        *Message        `json:"reply_to_message"`
	EditDate              uint64          `json:"edit_date"`
	Text                  string          `json:"text"`
	Entities              []MessageEntity `json:"entities"`
	Audio                 *Audio          `json:"audio"`
	Document              *Document       `json:"document"`
	Game                  *Game           `json:"game"`
	Photo                 []PhotoSize     `json:"photo"`
	Sticker               *Sticker        `json:"sticker"`
	Video                 *Video          `json:"video"`
	Voice                 *Voice          `json:"voice"`
	Caption               string          `json:"caption"`
	Contact               *Contact        `json:"contact"`
	Location              *Location       `json:"location"`
	Venue                 *Venue          `json:"venue"`
	NewChatMember         *User           `json:"new_chat_member"`
	LeftChatMember        *User           `json:"left_chat_member"`
	NewChatTitle          string          `json:"new_chat_title"`
	NewChatPhoto          []PhotoSize     `json:"new_chat_photo"`
	DeleteChatPhoto       bool            `json:"delete_chat_photo"`
	GroupChatCreated      bool            `json:"group_chat_created"`
	SupergroupChatCreated bool            `json:"supergroup_chat_created"`
	ChannelChatCreated    bool            `json:"channel_chat_created"`
	MigrateToChatID       ChatID          `json:"migrate_to_chat_id"`
	MigrateFromChatID     ChatID          `json:"migrate_from_chat_id"`
	PinnedMessage         *Message        `json:"pinned_message"`
}

type ReplyMarkup interface {
	itsReplyMarkup()
}

type replyMarkupImplementation struct{}

func (r replyMarkupImplementation) itsReplyMarkup() {}

// This object represents one button of the reply keyboard.
// For simple text buttons String can be used instead of this object to specify text of the button.
// Optional fields are mutually exclusive.
type KeyboardButton struct {
	Text string `json:"text"`

	// Optional
	RequestContact  bool `json:"request_contact,omitempty"`
	RequestLocation bool `json:"request_location,omitempty"`
}

// This object represents a custom keyboard with reply options
type ReplyKeyboardMarkup struct {
	replyMarkupImplementation
	Keyboard        [][]KeyboardButton `json:"keyboard"`
	ResizeKeyboard  bool               `json:"resize_keyboard,omitempty"`
	OneTimeKeyboard bool               `json:"one_time_keyboard,omitempty"`
	Selective       bool               `json:"selective,omitempty"`
}

// Upon receiving a message with this object,
// Telegram clients will remove the current custom keyboard and display the default letter-keyboard.
// By default, custom keyboards are displayed until a new keyboard is sent by a bot.
// An exception is made for one-time keyboards that are hidden immediately after the user presses a button
type ReplyKeyboardRemove struct {
	replyMarkupImplementation
	RemoveKeyboard bool `json:"remove_keyboard,omitempty"`
	Selective      bool `json:"selective,omitempty"`
}

// This object represents one button of an inline keyboard. You must use exactly one of the optional fields.
type InlineKeyboardButton struct {
	Text string `json:"text,omitempty"`

	// Optional
	URL               string `json:"url,omitempty"`
	CallbackData      string `json:"callback_data,omitempty"`
	SwitchInlineQuery string `json:"switch_inline_query,omitempty"`
}

// This object represents an inline keyboard that appears right next to the message it belongs to.
type InlineKeyboardMarkup struct {
	replyMarkupImplementation
	InlineKeyboard [][]InlineKeyboardButton `json:"inline_keyboard"`
}

// Upon receiving a message with this object,
// Telegram clients will display a reply interface to the user.
// This can be extremely useful if you want to create user-friendly step-by-step interfaces without having to sacrifice privacy mode.
type ForceReply struct {
	replyMarkupImplementation
	ForceReply bool `json:"force_reply"`
	Selective  bool `json:"selective,omitempty"`
}

// Represents a result of an inline query that was chosen by the user and sent to their chat partner.
type ChosenInlineResult struct {
	ResultID        string    `json:"result_id"`
	From            User      `json:"from"`
	Location        *Location `json:"location"`
	InlineMessageID string    `json:"inline_message_id"`
	Query           string    `json:"query"`
}

// This object represents an incoming callback query from a callback button in an inline keyboard.
// If the button that originated the query was attached to a message sent by the bot, the field message will be presented.
// If the button was attached to a message sent via the bot (in inline mode), the field inline_message_id will be presented.
type CallbackQuery struct {
	ID              string   `json:"id"`
	From            User     `json:"from"`
	Message         *Message `json:"message"`
	InlineMessageID string   `json:"inline_message_id"`
	Data            string   `json:"data"`
}

// This object represents an incoming inline query.
// When the user sends an empty query, your bot could return some default or trending results.
type InlineQuery struct {
	ID     string `json:"id"`
	From   User   `json:"from"`
	Query  string `json:"query"`
	Offset string `json:"offset"`
}

// This object represents an incoming update.
// Only one of the optional parameters can be present in any given update.
type Update struct {
	UpdateID uint64 `json:"update_id"`

	// Optional
	Message            *Message            `json:"message"`
	EditedMessage      *Message            `json:"edited_message"`
	ChannelPost        *Message            `json:"channel_post"`
	EditedChannelPost  *Message            `json:"edited_channel_post"`
	InlineQuery        *InlineQuery        `json:"inline_query"`
	ChosenInlineResult *ChosenInlineResult `json:"chosen_inline_result"`
	CallbackQuery      *CallbackQuery      `json:"callback_query"`
}

type WebhookInfo struct {
	URL                  string   `json:"url"`
	HasCustomCertificate bool     `json:"has_custom_certificate"`
	PendingUpdateCount   int      `json:"pending_update_count"`
	LastErrorDate        uint64   `json:"last_error_date"`
	LastErrorMessage     string   `json:"last_error_message"`
	MaxConnections       int      `json:"max_connections"`
	AllowedUpdates       []string `json:"allowed_updates"`
}
